/*
    \brief Deklarationen der Funktionen für den Active-Mode.

    \author Bernd Ottmann
    \date   21.02.2003
*/
#pragma once
#ifndef DF_ACTIVE_MODE_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define DF_ACTIVE_MODE_H
#endif

#include "df_defines.h"

#if (DF_API_VERSION >= 0x04010500)

/*!
    \brief Initalisiert und Startet das Bibiotheksmodule für den Active-Mode.

    Nach erfolgreicher Ausführung wird auf dem angegebenen Port auf eingehende Geräteverbindungen gewartet.\n
    Bei der entgegennahme werden entsprechende Informationen ermittelt und die Verbindung über eine ggf. dynamisch
    vergebene Verbindungsnummer oder über ein anzugebendes Mapping (DFCBindDeviceToChannel()) zugängelich gemacht.\n
    Die Annahme von "Datensatzmeldungen" ist standardmäßig aktiv und kann durch Aufruf von DFCSetRecordAvailable()
    wie gewünscht eingestellt werden.

    \ingroup active_mode

    \export{_DFCStartActiveConnection@36, DFCStartActiveConnection}

    \warning \li Diese Funktion öffnet einen Listen-Socket für die Entgegennahme von Verbindungen. Bitte stellen Sie sicher,
    dass Ihre Firewall - Konfiguration dieses zulässt und nicht blockiert.
    \li Der angegebene [Schnittstellentimeout](\ref dfc_read_timeout) bezieht sich auf alle entgegengenommenen
    Verbindungen und muss demnach so gewählt werden, dass die Verbindungsart mit den höchsten Paketlaufzeiten
    berücksichtigt wird. Betreiben Sie zum Beispiel Geräte im LAN sowie WAN (per GPRS) ist die Vorgabe für WAN (per GPRS) zu wählen.

    \note Für die Einrichtung des Active-Mode im Gerät, stehen Ihnen auch [Systemvariablen](\ref dfc_system_variables), speziell
    [Variablen für Active-Mode, Präfix COM](\ref dfcd_4_sysvar_com) zur Verfügung.

    \param[in] networkAddress Adresse der Netzwerkschnittstelle von der Verbindungen entgegengenommen werden sollen.
    Standardwert 0.0.0.0, dieses Bedeutet es wird von allen Netzwerkschnittstellen die Verbindungen
    entgegengenommen.
    \param[in] listenPort Port auf dem die Verbindungsanfragen einlaufen müssen. Standardwert 8000.
    \param[in] deviceAddress Die von der Bibliothek zu verwendende Geräteadresse. Standardwert ist 0.
    Hierbei handelt es sich nicht um die Geräteadresse des später anzusprechenden Gerätes. Diese
    Angabe hier wird als Absenderadresse in den Protokollpaketen verwendet.
    \param[in] timeout Zu verwendender [Schnittstellentimeout](\ref dfc_read_timeout) in Millisekunden. Standardwert 5000.
    \param[in] aliveTimeout Dieser Wert gibt in Sekunden an, in welchem Zyklus ein Datenpaket zwischen Gerät und
    Bibliothek gewechselt werden muss um sicherzustellen, dass die Verbindung noch besteht.\n
    Dieser Wert wird bei der Verbindungsentgegennahme auch an das Gerät weitergeleitet, damit
    dieses ebenfalls eine Verbindungsunterbrechung erkennt.
    \param[in] maxConnections Maximal anzunehmende Verbidnungen. Standardwert 20, Maximalwert 140.
    \param[in] infoFlags Es werden immer die Information über den Verbundenen Gerätetypen und dessen Seriennummer geliefert.\n
    Zusätzlich können Sie folgende Informationen anfordern. Diese werden bei der Verbindungsentgegennahme
    einmalig ermittelt und zwischengespeichert.
    <table class="paramvalues">
        <tr><th>Bitnummer</th><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>1</td><td>Firmwareversion (z. B. 04.03.010.PZE)</td></tr>
        <tr><td>2</td><td>2</td><td>IP Adresse und Port des Gerätes (z. B. 192.68.0.1:8000)</td></tr>
        <tr><td>3</td><td>4</td><td>MAC-Adresse (z.B. 00-20-4a-e7-1c-25)</td></tr>
        <tr><td>...</td><td></td><td>Nicht verwendet. Immer mit 0 übergeben.</td></tr>
    </table>
    Um zusätzlich die Informationen über Firmwareversion und MAC-Adresse abzurufen, muss entsprechend
    der Wert (1 + 4 =) 5 angegeben werden.
    Alle Angaben werden mit einem \\r getrennt in folgender fester Reihenfolge geliefert. Werte die nicht
    Ermittelt wurden, da ggf. nicht erwünscht, werden entspreched leer, jedoch nicht ausgelassen.
    <table class="paramvalues">
        <tr><th>Position</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>Geräteadresse anhand des Kürzels der Firmwareversion. Liste siehe DFCGetVersionFirmware()</td></tr>
        <tr><td>2</td><td>Seriennummer. Siehe DFCGetSeriennummer()</td></tr>
        <tr><td>3</td><td>Firmwareversion. (z. B. 04.03.010.PZE)</td></tr>
        <tr><td>4</td><td>IP Adresse und Port des Gerätes (z. B. 192.68.0.1:8000)</td></tr>
        <tr><td>5</td><td>MAC-Adresse (z.B. 00-20-4a-e7-1c-25)</td></tr>
    </table>

    \param[in] unused_setTo0 Reserviert. Standardwert 0.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.05 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCStartActiveConnection( const char *networkAddress, int listenPort, int deviceAddress, 
                                                                int timeout, int aliveTimeout, int maxConnections, int infoFlags, 
                                                                int unused_setTo0, int *errorNumber );

/*!
    \brief Stoppt das Bibiotheksmodule für den Active-Mode.

    Ggf. vorhandene Verbindungen des Active-Mode werden geschlossen. Alle gespeicherten Daten über die Verbindungen und
    derren aktuelle Zustände werden gelöscht.

    \ingroup active_mode

    \export{_DFCStopActiveConnection@4, DFCStopActiveConnection}

    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.05 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCStopActiveConnection( int *errorNumber );

/*!
    \brief Liefert die Erste Verbindungsnummer mit einer Verbindung im Active-Mode.

    Wurde mindesten eine Verbindung im Active-Mode entgegengenommen wird die zugeordnete Verbindung-Id geliefert.
    Der Wertebereich ist bei Anwendung von Mapping DFCBindDeviceToChannel() 1 - 250. Bei dynamischer Zuweisung
    entsprechend 251 - 391.

    \ingroup active_mode

    \export{_DFCGetFirstActiveConnectionId@0, DFCGetFirstActiveConnectionId}

    \returns Liefert -1 wenn keine Verbindung im Active-Mode besteht, ansonsten wird die Erste Verbindungsnummer zurückgegeben.

    \since 04.01.05 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetFirstActiveConnectionId();

/*!
    \brief Liefert die Erste Verbindungsnummer mit einer Verbindung im Active-Mode.

    Wurde mindesten eine Verbindung im Active-Mode entgegengenommen wird die zugeordnete Verbindung-Id geliefert.
    Der Wertebereich ist bei Anwendung von Mapping DFCBindDeviceToChannel() 1 - 250. Bei dynamischer Zuweisung
    entsprechend 251 - 391.

    \ingroup active_mode

    \export{_DFCGetFirstActiveChannelID@0, DFCGetFirstActiveChannelID}

    \returns Liefert -1 wenn keine Verbindung im Active-Mode besteht, ansonsten wird die Erste Verbindungsnummer zurückgegeben.

    \since 04.01.05 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetFirstActiveChannelID();

/*!
    \brief Liefert die Nächste Verbindungsnummer mit einer Verbindung im Active-Mode.

    Es wird eine ggf. folgende Verbindungsnummer mit einer Verbindung im Active-Mode gesucht und zurückgeliefert.
    Der Wertebereich ist bei Anwendung von Mapping DFCBindDeviceToChannel() 1 - 250. Bei dynamischer Zuweisung
    entsprechend 251 - 391.

    \ingroup active_mode

    \export{_DFCGetNextActiveConnectionId@4, DFCGetNextActiveConnectionId}

    \returns Liefert -1 wenn keine weitere Verbindung im Active-Mode besteht, ansonsten wird die Nächste Verbindungsnummer zurückgegeben.

    \since 04.01.05 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetNextActiveConnectionId( int previousNumber );

/*!
    \brief Liefert die Nächste Verbindungsnummer mit einer Verbindung im Active-Mode.

    Es wird eine ggf. folgende Verbindungsnummer mit einer Verbindung im Active-Mode gesucht und zurückgeliefert.
    Der Wertebereich ist bei Anwendung von Mapping DFCBindDeviceToChannel() 1 - 250. Bei dynamischer Zuweisung
    entsprechend 251 - 391.

    \ingroup active_mode

    \export{_DFCGetNextActiveChannelID@4, DFCGetNextActiveChannelID}

    \returns Liefert -1 wenn keine weitere Verbindung im Active-Mode besteht, ansonsten wird die Nächste Verbindungsnummer zurückgegeben.

    \since 04.01.05 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetNextActiveChannelID( int previousNumber );

/*!
    \brief Liefert Informationen zu einer Verbindung im Active-Mode.

    Es werden alle ermittelten und hinterlegten Werte zur Verbindung geliefert.\n
    Beim Start des Active-Mode mit DFCStartActiveConnection() können zusätzlich zu liefernde Werte angegeben werden.

    \ingroup active_mode

    \export{_DFCGetInfoActiveChannel@12, DFCGetInfoActiveChannel}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[out] value Es werden immer die Information über den Verbundenen Gerätetypen und dessen Seriennummer geliefert.\n
    Zusätzlich werden die angeforderten Werte aus der Angabe bei DFCStartActiveConnection() geliefert.\n
    Alle Angaben werden mit einem \\r getrennt in folgender fester Reihenfolge geliefert. Werte die nicht
    Ermittelt wurden, da ggf. nicht erwünscht, werden entspreched leer, jedoch nicht ausgelassen.
    <table class="paramvalues">
        <tr><th>Position</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>Geräteadresse anhand des Kürzels der Firmwareversion. Liste siehe DFCGetVersionFirmware()</td></tr>
        <tr><td>2</td><td>Seriennummer. Siehe DFCGetSeriennummer()</td></tr>
        <tr><td>3</td><td>Firmwareversion. (z. B. 04.03.010.PZE)</td></tr>
        <tr><td>4</td><td>IP Adresse und Port des Gerätes (z. B. 192.68.0.1:8000)</td></tr>
        <tr><td>5</td><td>MAC-Adresse (z.B. 00-20-4a-e7-1c-25)</td></tr>
    </table>
    \param[in,out] valueLength Als \c in die maximale Länge von \a value, als \c out die in \a value übergebene Zeichenfolge.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.05 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetInfoActiveChannel( int connectionNumber, char *value, int *valueLength );

/*!
    \brief Legt fest ob die Bibliothek die Ereignismeldung "Datensatz vorhanden" auswertet und durch die Funktion
    DFCRecordAvailable() zur Verfügung stellen soll.

    Nach Ausführung von DFCStartActiveConnection() ist die Annahme und Meldung von Datensatzmeldungen aktiviert. Über diese
    Funktion kann es Bibliotheksseitig deaktiviert werden.

    \ingroup active_mode

    \export{_DFCSetRecordAvailable@4, DFCSetRecordAvailable}

    \param[in] enabled Bei Angabe von 0 werden eingehende Datensatzmeldungen ignoriert. Vorhandene zwischengespeicherte
    werden entsprechend gelöscht. Bei Angabe von 1 werden eingehenden Datensatzmeldungen für den Abruf
    mittels DFCRecordAvailable() zwischengepsichert.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.05 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetRecordAvailable( int enabled );

/*!
    \brief Liefert die nächste Verbindung im Active-Mode welche Datensätze gemeldet hat.

    Es werden alle ermittelten und hinterlegten Werte zur Verbindung geliefert.\n
    Beim Start des Active-Mode mit DFCStartActiveConnection() können zusätzlich zu liefernde Werte angegeben werden.

    \ingroup active_mode

    \export{_DFCRecordAvailable@20, DFCRecordAvailable}

    \param[out] connectionNumber Nächste Verbindungsnummer, welche Datensätze gemeldet hat.
    \param[out] deviceAddress Nächste Geräteadresse, welche Datensätze gemeldet hat.
    \param[out] infoText Diese Angabe ist die gleiche wie beim Aufruf von DFCGetInfoActiveChannel(). Siehe dort.
    \param[in,out] infoTextLength Als \c in die maximale Länge von \a infoText, als \c out die in \a infoText übergebene Zeichenfolge.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.05 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCRecordAvailable( int *connectionNumber, int *deviceAddress, char *infoText, int *infoTextLength, int *errorNumber );

/*!
    \brief Bindet ein Gerät im Active-Mode anhand von Gerätetyp und Seriennummer oder Geräte-Ip an eine vorgegebene Verbindungsnummer.

    Die Funktion kann vor Ausführung von DFCStartActiveConnection() oder zur Laufzeit aufgerufen werden. Durch sie können
    Verbindungen im Active-Mode an feste Verbindungsnummern geknüpft werden.\n
    Über die Funktion DFCIsChannelOpen() kann geprüft werden ob eine Verbindung besteht. Die gebundenen Verbidnungs-Id's
    tauchen auch bei einem Abruf mittels DFCGetFirstActiveChannelID() und DFCGetNextActiveChannelID() auf.

    \ingroup active_mode

    \export{_DFCBindDeviceToChannel@20, DFCBindDeviceToChannel}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer. Wertebereich 1 - 250.
    \param[in] deviceType \li Geräteadresse, Liste siehe DFCGetVersionFirmware(). Zu dieser Angabe ist auch die Angabe der
    Seriennummer (deviceSerial) notwendig. Da nur beide Angaben eine eindeutige Geräteidentifikation
    darstellen.\n
    \li Wird eine -1 angegeben und es liegt keine Angabe zu deviceIp vor, entfernt dieses eine
    vorhandene Gerätebindung zu der Verbindungsnummer.
    \param[in] deviceSerial Seriennummer des Gerätes.
    \param[in] deviceIp Ip-Adresse des zu bindenden Gerätes oder \a nullptr wenn eine Bindung über Geräteadresse und Seriennummer
    erfolgen soll.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.05 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCBindDeviceToChannel( int connectionNumber, int deviceType, int deviceSerial, const char *deviceIp, int *errorNumber );

#endif

#endif
